@@ -1,5 +1,5 @@
 #!/bin/sh
-# Copyright (c) 2019-2024, The Khronos Group Inc.
+# Copyright (c) 2019-2025 The Khronos Group Inc.
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -135,6 +135,7 @@ getDocsFilenames() {
         | grep -v "CMakeLists.txt" \
         | grep -v "ubmitted" \
         | grep -v "experimental" \
+        | grep -v "xml\.reservations" \
         | grep -v "compare" \
         | grep -v "JP\.jpg"
 }
@@ -166,9 +167,12 @@ getSDKSourceFilenames() {
         .azure-pipelines/openxr-sdk-source.yml \
         .github/dependabot.yml \
         .github/scripts \
+        .github/.java-version* \
         .github/workflows/android.yml \
         .github/workflows/check_clang_format_and_codespell.yml \
         .github/workflows/gradle-wrapper-validation.yml \
+        .github/workflows/macos-build.yml \
+        .github/workflows/macOS.yml \
         .github/workflows/msvc-build-preset.yml \
         .github/workflows/pr.yml \
         .github/workflows/release.yml \
@@ -206,10 +210,12 @@ getSDKSourceFilenames() {
         src/common_config.h.in \
         src/external/CMakeLists.txt \
         src/external/android-jni-wrappers \
+        src/external/glad2 \
         src/external/jnipp \
         src/external/jsoncpp \
         src/external/catch2 \
         src/external/metal-cpp \
+        src/external/sanitizers-cmake \
         src/loader \
         src/scripts \
         src/tests \
@@ -237,6 +243,7 @@ getSDKFilenames() {
         .azure-pipelines/nuget \
         .azure-pipelines/openxr-sdk.yml \
         .github/scripts \
+        .github/.java-version* \
         .github/workflows/android.yml \
         .github/workflows/check_clang_format_and_codespell.yml \
         .github/workflows/msvc-build-preset.yml \
@@ -278,6 +285,7 @@ getConformanceFilenames() {
         runClangFormat.sh \
         tox.ini \
         .github/dependabot.yml \
+        .github/.java-version* \
         .github/scripts \
         .github/workflows/android-cts-build.yml \
         .github/workflows/android-cts-pr.yml \
